@@ -1,11 +1,28 @@
+/*
+ * Copyright (C) 2014 Stuart Howarth <showarth@marxoft.co.uk>
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms and conditions of the GNU Lesser General Public License,
+ * version 3, as published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope it will be useful, but WITHOUT ANY
+ * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+ * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
+ * more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
+ */
+
 #ifndef DOWNLOADSETTINGSTAB_H
 #define DOWNLOADSETTINGSTAB_H
 
 #include <QWidget>
 
+class ArchivePasswordsModel;
 class QLineEdit;
 class QCheckBox;
-class QStringListModel;
 class QListView;
 class QPushButton;
 class QMenu;
@@ -30,7 +47,6 @@ private slots:
     void onPasswordEditTextChanged(const QString &text);
     void addArchivePassword();
     void removeArchivePassword();
-    void onArchivePasswordsChanged();
     
 private:
     QLineEdit *m_pathEdit;
@@ -39,7 +55,7 @@ private:
     QCheckBox *m_extractCheckbox;
     QCheckBox *m_subfoldersCheckbox;
     QCheckBox *m_deleteCheckbox;
-    QStringListModel *m_model;
+    ArchivePasswordsModel *m_model;
     QListView *m_view;
     QMenu *m_contextMenu;
     QAction *m_removePasswordAction;
