@@ -29,6 +29,7 @@
 #include "../shared/urlretriever.h"
 #include "../shared/pluginmanager.h"
 #include "../shared/transfermodel.h"
+#include "../shared/transferfiltermodel.h"
 #include "../shared/transfer.h"
 #include "../shared/settings.h"
 #include "../shared/clipboardmonitor.h"
@@ -50,6 +51,9 @@
 #include <QTimer>
 #include <QCoreApplication>
 #include <QMessageBox>
+#ifdef TABLE_TRANSFER_VIEW
+#include <QHeaderView>
+#endif
 #if QT_VERSION >= 0x050000
 #include <QStandardPaths>
 #else
@@ -77,31 +81,47 @@ MainWindow::MainWindow(QWidget *parent) :
     m_activeTransfersLabel(new QLabel(QString("0 DLs"), this)),
     m_speedLabel(new QLabel("0 kB/s", this)),
     m_model(TransferModel::instance()),
+    m_filterModel(new TransferFilterModel(this)),
     m_view(new QTreeView(this)),
     m_urlMenu(this->menuBar()->addMenu(tr("URLs"))),
-    m_addUrlsAction(m_urlMenu->addAction(QIcon::fromTheme("list-add"), tr("Add URLs"), this, SLOT(showAddUrlsDialog()), Qt::CTRL + Qt::Key_A)),
-    m_importUrlsAction(m_urlMenu->addAction(QIcon::fromTheme("document-open"), tr("Import URLs"), this, SLOT(showTextFileDialog()), Qt::CTRL + Qt::Key_I)),
-    m_retrieveUrlsAction(m_urlMenu->addAction(QIcon::fromTheme("applications-internet"), tr("Retrieve URLs"), this, SLOT(showRetrieveUrlsDialog()), Qt::CTRL + Qt::Key_R)),
-    m_quitAction(m_urlMenu->addAction(QIcon::fromTheme("system-shutdown"), tr("Quit"), QCoreApplication::instance(), SLOT(quit()), Qt::CTRL + Qt::Key_Q)),
+    m_addUrlsAction(m_urlMenu->addAction(QIcon::fromTheme("list-add"), tr("Add URLs"),
+                                         this, SLOT(showAddUrlsDialog()), Qt::CTRL + Qt::Key_A)),
+    m_importUrlsAction(m_urlMenu->addAction(QIcon::fromTheme("document-open"), tr("Import URLs"),
+                                            this, SLOT(showTextFileDialog()), Qt::CTRL + Qt::Key_I)),
+    m_retrieveUrlsAction(m_urlMenu->addAction(QIcon::fromTheme("applications-internet"), tr("Retrieve URLs"),
+                                              this, SLOT(showRetrieveUrlsDialog()), Qt::CTRL + Qt::Key_R)),
+    m_quitAction(m_urlMenu->addAction(QIcon::fromTheme("system-shutdown"), tr("Quit"),
+                                      QCoreApplication::instance(), SLOT(quit()), Qt::CTRL + Qt::Key_Q)),
     m_transferMenu(this->menuBar()->addMenu(tr("Download"))),
-    m_transferPropertiesAction(m_transferMenu->addAction(QIcon::fromTheme("document-properties"), tr("Properties"), this, SLOT(showCurrentTransferProperties()))),
-    m_transferConvertToAudioAction(m_transferMenu->addAction(tr("Convert to audio file"), this, SLOT(setConvertCurrentTransferToAudio()))),
-    m_transferStartAction(m_transferMenu->addAction(QIcon::fromTheme("media-playback-start"), tr("Start"), this, SLOT(startCurrentTransfer()))),
-    m_transferPauseAction(m_transferMenu->addAction(QIcon::fromTheme("media-playback-pause"), tr("Pause"), this, SLOT(pauseCurrentTransfer()))),
+    m_transferPropertiesAction(m_transferMenu->addAction(QIcon::fromTheme("document-properties"), tr("Properties"),
+                                                         this, SLOT(showCurrentTransferProperties()))),
+    m_transferConvertToAudioAction(m_transferMenu->addAction(tr("Convert to audio file"),
+                                                             this, SLOT(setConvertCurrentTransferToAudio()))),
+    m_transferStartAction(m_transferMenu->addAction(QIcon::fromTheme("media-playback-start"), tr("Start"),
+                                                    this, SLOT(startCurrentTransfer()))),
+    m_transferPauseAction(m_transferMenu->addAction(QIcon::fromTheme("media-playback-pause"), tr("Pause"),
+                                                    this, SLOT(pauseCurrentTransfer()))),
     m_transferConnectionsMenu(m_transferMenu->addMenu(tr("Connections"))),
     m_transferConnectionsGroup(new QActionGroup(this)),
     m_transferCategoryMenu(m_transferMenu->addMenu(tr("Category"))),
     m_transferCategoryGroup(new QActionGroup(this)),
     m_transferPriorityMenu(m_transferMenu->addMenu(tr("Priority"))),
     m_transferPriorityGroup(new QActionGroup(this)),
-    m_transferHighPriorityAction(m_transferPriorityMenu->addAction(tr("High"), this, SLOT(setCurrentTransferPriority()))),
-    m_transferNormalPriorityAction(m_transferPriorityMenu->addAction(tr("Normal"), this, SLOT(setCurrentTransferPriority()))),
-    m_transferLowPriorityAction(m_transferPriorityMenu->addAction(tr("Low"), this, SLOT(setCurrentTransferPriority()))),
-    m_transferRemoveAction(m_transferMenu->addAction(QIcon::fromTheme("edit-delete"), tr("Remove"), this, SLOT(removeCurrentTransfer()))),
+    m_transferHighPriorityAction(m_transferPriorityMenu->addAction(tr("High"),
+                                                                   this, SLOT(setCurrentTransferPriority()))),
+    m_transferNormalPriorityAction(m_transferPriorityMenu->addAction(tr("Normal"),
+                                                                     this, SLOT(setCurrentTransferPriority()))),
+    m_transferLowPriorityAction(m_transferPriorityMenu->addAction(tr("Low"),
+                                                                  this, SLOT(setCurrentTransferPriority()))),
+    m_transferRemoveAction(m_transferMenu->addAction(QIcon::fromTheme("edit-delete"), tr("Remove"),
+                                                     this, SLOT(removeCurrentTransfer()))),
     m_packageMenu(this->menuBar()->addMenu(tr("Package"))),
-    m_packagePropertiesAction(m_packageMenu->addAction(QIcon::fromTheme("document-properties"), tr("Properties"), this, SLOT(showCurrentPackageProperties()))),
-    m_packageStartAction(m_packageMenu->addAction(QIcon::fromTheme("media-playback-start"), tr("Start"), this, SLOT(startCurrentPackage()))),
-    m_packagePauseAction(m_packageMenu->addAction(QIcon::fromTheme("media-playback-pause"), tr("Pause"), this, SLOT(pauseCurrentPackage()))),
+    m_packagePropertiesAction(m_packageMenu->addAction(QIcon::fromTheme("document-properties"), tr("Properties"),
+                                                       this, SLOT(showCurrentPackageProperties()))),
+    m_packageStartAction(m_packageMenu->addAction(QIcon::fromTheme("media-playback-start"), tr("Start"),
+                                                  this, SLOT(startCurrentPackage()))),
+    m_packagePauseAction(m_packageMenu->addAction(QIcon::fromTheme("media-playback-pause"), tr("Pause"),
+                                                  this, SLOT(pauseCurrentPackage()))),
     m_packageCategoryMenu(m_packageMenu->addMenu(tr("Category"))),
     m_packageCategoryGroup(new QActionGroup(this)),
     m_packagePriorityMenu(m_packageMenu->addMenu(tr("Priority"))),
@@ -109,9 +129,11 @@ MainWindow::MainWindow(QWidget *parent) :
     m_packageHighPriorityAction(m_packagePriorityMenu->addAction(tr("High"), this, SLOT(setCurrentPackagePriority()))),
     m_packageNormalPriorityAction(m_packagePriorityMenu->addAction(tr("Normal"), this, SLOT(setCurrentPackagePriority()))),
     m_packageLowPriorityAction(m_packagePriorityMenu->addAction(tr("Low"), this, SLOT(setCurrentPackagePriority()))),
-    m_packageRemoveAction(m_packageMenu->addAction(QIcon::fromTheme("edit-delete"), tr("Remove"), this, SLOT(removeCurrentPackage()))),
+    m_packageRemoveAction(m_packageMenu->addAction(QIcon::fromTheme("edit-delete"), tr("Remove"),
+                                                   this, SLOT(removeCurrentPackage()))),
     m_editMenu(this->menuBar()->addMenu(tr("Edit"))),
-    m_preferencesAction(m_editMenu->addAction(QIcon::fromTheme("preferences-desktop"), tr("Preferences"), this, SLOT(showSettingsDialog()), Qt::CTRL + Qt::Key_P)),
+    m_preferencesAction(m_editMenu->addAction(QIcon::fromTheme("preferences-desktop"), tr("Preferences"),
+                                              this, SLOT(showSettingsDialog()), Qt::CTRL + Qt::Key_P)),
     m_helpMenu(this->menuBar()->addMenu(tr("Help"))),
     m_aboutAction(m_helpMenu->addAction(QIcon::fromTheme("help-about"), tr("About"), this, SLOT(showAboutDialog()))),
     m_checkDialog(new CheckUrlsDialog(this)),
@@ -127,11 +149,16 @@ MainWindow::MainWindow(QWidget *parent) :
 {
     this->setWindowTitle("QDL");
     this->setCentralWidget(m_view);
-    this->setMinimumSize(600, 600);
     this->setAcceptDrops(true);
+#ifdef TABLE_TRANSFER_VIEW
+    this->setMinimumSize(1100, 600);
+#else
+    this->setMinimumSize(600, 600);
+#endif
 
     for (int i = 1; i <= MAX_CONCURRENT_TRANSFERS; i++) {
-        QAction *action = m_concurrentMenu->addAction(QString::number(i), this, SLOT(setMaximumConcurrentTransfers()));
+        QAction *action = m_concurrentMenu->addAction(QString::number(i),
+                                                      this, SLOT(setMaximumConcurrentTransfers()));
         action->setData(i);
         action->setCheckable(true);
 
@@ -143,8 +170,10 @@ MainWindow::MainWindow(QWidget *parent) :
     }
 
     for (int i = 1; i <= MAX_CONNECTIONS; i++) {
-        QAction *action = m_transferConnectionsMenu->addAction(QString::number(i), this, SLOT(setCurrentTransferConnections()));
-        QAction *globalAction = m_connectionsMenu->addAction(QString::number(i), this, SLOT(setGlobalTransferConnections()));
+        QAction *action = m_transferConnectionsMenu->addAction(QString::number(i),
+                                                               this, SLOT(setCurrentTransferConnections()));
+        QAction *globalAction = m_connectionsMenu->addAction(QString::number(i),
+                                                             this, SLOT(setGlobalTransferConnections()));
         action->setData(i);
         action->setCheckable(true);
         globalAction->setData(i);
@@ -166,7 +195,8 @@ MainWindow::MainWindow(QWidget *parent) :
     m_rateLimitMenu->addSeparator();
 
     for (int i = 1; i < RATE_LIMITS.size(); i++) {
-        QAction *action = m_rateLimitMenu->addAction(QString::number(RATE_LIMITS.at(i) / 1000) + " kB/s", this, SLOT(setDownloadRateLimit()));
+        QAction *action = m_rateLimitMenu->addAction(QString::number(RATE_LIMITS.at(i) / 1000) + " kB/s",
+                                                     this, SLOT(setDownloadRateLimit()));
         action->setData(RATE_LIMITS.at(i));
         action->setCheckable(true);
 
@@ -182,7 +212,6 @@ MainWindow::MainWindow(QWidget *parent) :
     m_filterComboBox->setModel(new StatusFilterModel(m_filterComboBox));
 
     m_nextActionComboBox->setModel(new TransferActionModel(m_nextActionComboBox));
-    m_nextActionComboBox->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Maximum);
 
     m_actionToolBar->setAllowedAreas(Qt::TopToolBarArea);
     m_actionToolBar->setMovable(false);
@@ -203,7 +232,9 @@ MainWindow::MainWindow(QWidget *parent) :
     m_filterToolBar->addWidget(m_searchEdit);
 
     this->addToolBar(m_actionToolBar);
+#ifndef TABLE_TRANSFER_VIEW
     this->addToolBarBreak();
+#endif
     this->addToolBar(m_filterToolBar);
 
     m_optionsButton->setIcon(QIcon::fromTheme("document-properties"));
@@ -274,16 +305,33 @@ MainWindow::MainWindow(QWidget *parent) :
     m_packageLowPriorityAction->setCheckable(true);
     m_packageLowPriorityAction->setActionGroup(m_packagePriorityGroup);
 
-    m_view->setModel(m_model);
+    m_view->setModel(m_filterModel);
     m_view->setAlternatingRowColors(true);
-    m_view->setHeaderHidden(true);
     m_view->setSelectionBehavior(QTreeView::SelectRows);
     m_view->setContextMenuPolicy(Qt::CustomContextMenu);
     m_view->setEditTriggers(QTreeView::NoEditTriggers);
+    m_view->setDragEnabled(true);
+    m_view->setAcceptDrops(true);
+    m_view->setDropIndicatorShown(true);
+    m_view->setDragDropMode(QTreeView::InternalMove);
+    m_view->setDefaultDropAction(Qt::MoveAction);
     m_view->setExpandsOnDoubleClick(true);
     m_view->setItemsExpandable(true);
     m_view->setIndentation(16);
+    m_view->setUniformRowHeights(true);
+    m_view->setAllColumnsShowFocus(true);
     m_view->setItemDelegate(new TransferItemDelegate(m_view));
+#ifdef TABLE_TRANSFER_VIEW
+    QHeaderView *header = m_view->header();
+    QFontMetrics fm = header->fontMetrics();
+    header->resizeSection(0, 350);
+    header->resizeSection(1, fm.width("A long category name") + 20);
+    header->resizeSection(2, fm.width(m_model->headerData(2).toString()) + 20);
+    header->resizeSection(3, fm.width(m_model->headerData(3).toString()) + 20);
+    header->resizeSection(4, fm.width("999.99MB of 999.99MB (99.99%)") + 20);
+#else
+    m_view->setHeaderHidden(true);
+#endif
 
     m_progressDialog->setWindowTitle(tr("Please wait"));
 
@@ -311,7 +359,7 @@ MainWindow::MainWindow(QWidget *parent) :
 
     this->connect(m_startAction, SIGNAL(triggered()), m_model, SLOT(start()));
     this->connect(m_pauseAction, SIGNAL(triggered()), m_model, SLOT(pause()));
-    this->connect(m_searchEdit, SIGNAL(textChanged(QString)), m_model, SLOT(setSearchQuery(QString)));
+    this->connect(m_searchEdit, SIGNAL(textChanged(QString)), m_filterModel, SLOT(setSearchQuery(QString)));
     this->connect(m_filterComboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(onFilterBoxIndexChanged(int)));
     this->connect(m_nextActionComboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(onNextActionIndexChanged(int)));
     this->connect(m_view, SIGNAL(customContextMenuRequested(QPoint)), this, SLOT(showContextMenu(QPoint)));
@@ -319,18 +367,24 @@ MainWindow::MainWindow(QWidget *parent) :
     this->connect(m_packageMenu, SIGNAL(aboutToShow()), this, SLOT(setPackageMenuActions()));
     this->connect(m_optionsButton, SIGNAL(clicked()), this, SLOT(showOptionsMenu()));
     this->connect(m_model, SIGNAL(countChanged(int)), this, SLOT(onPackageCountChanged(int)));
-    this->connect(m_model, SIGNAL(nextActionChanged(Transfers::Action)), this, SLOT(onNextActionChanged(Transfers::Action)));
+    this->connect(m_model, SIGNAL(nextActionChanged(Transfers::Action)),
+                  this, SLOT(onNextActionChanged(Transfers::Action)));
     this->connect(m_trayAddUrlsAction, SIGNAL(triggered()), this, SLOT(showAddUrlsDialog()));
     this->connect(m_trayImportUrlsAction, SIGNAL(triggered()), this, SLOT(showTextFileDialog()));
     this->connect(m_trayRetrieveUrlsAction, SIGNAL(triggered()), this, SLOT(showRetrieveUrlsDialog()));
     this->connect(m_trayPreferencesAction, SIGNAL(triggered()), this, SLOT(showSettingsDialog()));
     this->connect(m_trayQuitAction, SIGNAL(triggered()), QCoreApplication::instance(), SLOT(quit()));
-    this->connect(m_trayIcon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)), this, SLOT(onTrayIconActivated(QSystemTrayIcon::ActivationReason)));
-    this->connect(Settings::instance(), SIGNAL(maximumConcurrentTransfersChanged(int,int)), this, SLOT(onMaximumConcurrentTransfersChanged(int, int)));
-    this->connect(Settings::instance(), SIGNAL(maximumConnectionsPerTransferChanged(int,int)), this, SLOT(onGlobalTransferConnectionsChanged(int, int)));
-    this->connect(Settings::instance(), SIGNAL(downloadRateLimitChanged(int)), this, SLOT(onDownloadRateLimitChanged(int)));
+    this->connect(m_trayIcon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)),
+                  this, SLOT(onTrayIconActivated(QSystemTrayIcon::ActivationReason)));
+    this->connect(Settings::instance(), SIGNAL(maximumConcurrentTransfersChanged(int,int)),
+                  this, SLOT(onMaximumConcurrentTransfersChanged(int, int)));
+    this->connect(Settings::instance(), SIGNAL(maximumConnectionsPerTransferChanged(int,int)),
+                  this, SLOT(onGlobalTransferConnectionsChanged(int, int)));
+    this->connect(Settings::instance(), SIGNAL(downloadRateLimitChanged(int)),
+                  this, SLOT(onDownloadRateLimitChanged(int)));
     this->connect(Database::instance(), SIGNAL(categoriesChanged()), this, SLOT(setCategoryMenuActions()));
-    this->connect(ClipboardMonitor::instance(), SIGNAL(clipboardUrlsReady(QString)), this, SLOT(showAddUrlsDialog(QString)));
+    this->connect(ClipboardMonitor::instance(), SIGNAL(clipboardUrlsReady(QString)),
+                  this, SLOT(showAddUrlsDialog(QString)));
     this->connect(PluginManager::instance(), SIGNAL(busy(QString,int)), this, SLOT(showProgressDialog(QString,int)));
     this->connect(PluginManager::instance(), SIGNAL(progressChanged(int)), this, SLOT(updateProgressDialog(int)));
     this->connect(PluginManager::instance(), SIGNAL(pluginsReady()), this, SLOT(onPluginsReady()));
@@ -414,7 +468,7 @@ void MainWindow::onPluginsReady() {
 }
 
 void MainWindow::onFilterBoxIndexChanged(int index) {
-    m_model->setStatusFilter(static_cast<Transfers::Status>(m_filterComboBox->itemData(index, Qt::UserRole + 1).toInt()));
+    m_filterModel->setStatusFilter(static_cast<Transfers::Status>(m_filterComboBox->itemData(index, Qt::UserRole + 1).toInt()));
 }
 
 void MainWindow::onNextActionIndexChanged(int index) {
@@ -426,13 +480,27 @@ void MainWindow::onNextActionChanged(Transfers::Action action) {
 }
 
 void MainWindow::onPackageCountChanged(int count) {
-    m_transferMenu->setEnabled(count > 0);
-    m_packageMenu->setEnabled(count > 0);
-    m_startAction->setEnabled(count > 0);
-    m_pauseAction->setEnabled(count > 0);
-    m_nextActionComboBox->setEnabled(count > 0);
-    m_filterComboBox->setEnabled(count > 0);
-    m_searchEdit->setEnabled(count > 0);
+    if (count > 0) {
+        m_view->setContextMenuPolicy(Qt::CustomContextMenu);
+        m_transferMenu->setEnabled(true);
+        m_packageMenu->setEnabled(true);
+        m_startAction->setEnabled(true);
+        m_pauseAction->setEnabled(true);
+        m_nextActionComboBox->setEnabled(true);
+        m_filterComboBox->setEnabled(true);
+        m_searchEdit->setEnabled(true);
+    }
+    else {
+        m_view->setContextMenuPolicy(Qt::NoContextMenu);
+        m_transferMenu->setEnabled(false);
+        m_packageMenu->setEnabled(false);
+        m_startAction->setEnabled(false);
+        m_pauseAction->setEnabled(false);
+        m_nextActionComboBox->setEnabled(false);
+        m_filterComboBox->setEnabled(false);
+        m_searchEdit->setEnabled(false);
+        m_searchEdit->clear();
+    }
 }
 
 void MainWindow::showOptionsMenu() {
@@ -501,7 +569,8 @@ void MainWindow::updateActiveTransfers(int active) {
 }
 
 void MainWindow::updateWindowTitle() {
-    this->setWindowTitle(QString("QDL | %1 DLs | %2 kB/s").arg(m_model->activeTransfers()).arg(m_model->totalDownloadSpeed()));
+    this->setWindowTitle(QString("QDL | %1 DLs | %2 kB/s").arg(m_model->activeTransfers())
+                                                          .arg(m_model->totalDownloadSpeed()));
 }
 
 void MainWindow::setTransferMenuActions() {
@@ -512,11 +581,10 @@ void MainWindow::setTransferMenuActions() {
     }
 
     m_transferConvertToAudioAction->setEnabled(index.data(Transfer::ConvertibleToAudioRole).toBool());
-    m_transferConvertToAudioAction->setChecked((m_transferConvertToAudioAction->isEnabled()) && (index.data(Transfer::ConvertToAudioRole).toBool()));
+    m_transferConvertToAudioAction->setChecked((m_transferConvertToAudioAction->isEnabled())
+                                                && (index.data(Transfer::ConvertToAudioRole).toBool()));
 
-    Transfers::Status status = static_cast<Transfers::Status>(index.data(Transfer::StatusRole).toInt());
-
-    switch (status) {
+    switch (index.data(Transfer::StatusRole).toInt()) {
     case Transfers::Paused:
     case Transfers::Failed:
         m_transferStartAction->setEnabled(true);
@@ -527,8 +595,8 @@ void MainWindow::setTransferMenuActions() {
         m_transferPauseAction->setEnabled(true);
     }
 
-    int preferredConnections = index.data(Transfer::PreferredConnectionsRole).toInt();
-    int maximumConnections = index.data(Transfer::MaximumConnectionsRole).toInt();
+    const int preferredConnections = index.data(Transfer::PreferredConnectionsRole).toInt();
+    const int maximumConnections = index.data(Transfer::MaximumConnectionsRole).toInt();
 
     for (int i = 0; i < m_transferConnectionsMenu->actions().size(); i++) {
         m_transferConnectionsMenu->actions().at(i)->setEnabled(i < maximumConnections);
@@ -538,9 +606,7 @@ void MainWindow::setTransferMenuActions() {
         m_transferConnectionsMenu->actions().at(preferredConnections - 1)->setChecked(true);
     }
 
-    Transfers::Priority priority = static_cast<Transfers::Priority>(index.data(Transfer::PriorityRole).toInt());
-
-    switch (priority) {
+    switch (index.data(Transfer::PriorityRole).toInt()) {
     case Transfers::HighPriority:
         m_transferHighPriorityAction->setChecked(true);
         break;
@@ -576,9 +642,7 @@ void MainWindow::setPackageMenuActions() {
         index = index.parent();
     }
 
-    Transfers::Status status = static_cast<Transfers::Status>(index.data(Transfer::StatusRole).toInt());
-
-    switch (status) {
+    switch (index.data(Transfer::StatusRole).toInt()) {
     case Transfers::Paused:
     case Transfers::Failed:
         m_packageStartAction->setEnabled(true);
@@ -589,9 +653,7 @@ void MainWindow::setPackageMenuActions() {
         m_packagePauseAction->setEnabled(true);
     }
 
-    Transfers::Priority priority = static_cast<Transfers::Priority>(index.data(Transfer::PriorityRole).toInt());
-
-    switch (priority) {
+    switch (index.data(Transfer::PriorityRole).toInt()) {
     case Transfers::HighPriority:
         m_packageHighPriorityAction->setChecked(true);
         break;
@@ -642,39 +704,39 @@ void MainWindow::showContextMenu(const QPoint &pos) {
 
 void MainWindow::showCurrentTransferProperties() {
     if (m_view->currentIndex().isValid()) {
-        TransferPropertiesDialog *dialog = new TransferPropertiesDialog(m_model->get(m_view->currentIndex()), this);
+        TransferPropertiesDialog *dialog = new TransferPropertiesDialog(m_model->get(m_filterModel->mapToSource(m_view->currentIndex())), this);
         dialog->open();
     }
 }
 
 void MainWindow::setConvertCurrentTransferToAudio() {
     if (m_view->currentIndex().isValid()) {
-        m_model->setData(m_view->currentIndex(), m_transferConvertToAudioAction->isChecked(), Transfer::ConvertToAudioRole);
+        m_model->setData(m_filterModel->mapToSource(m_view->currentIndex()), m_transferConvertToAudioAction->isChecked(), Transfer::ConvertToAudioRole);
     }
 }
 
 void MainWindow::startCurrentTransfer() {
     if (m_view->currentIndex().isValid()) {
-        m_model->setData(m_view->currentIndex(), Transfers::Queued, Transfer::StatusRole);
+        m_model->setData(m_filterModel->mapToSource(m_view->currentIndex()), Transfers::Queued, Transfer::StatusRole);
     }
 }
 
 void MainWindow::pauseCurrentTransfer() {
     if (m_view->currentIndex().isValid()) {
-        m_model->setData(m_view->currentIndex(), Transfers::Paused, Transfer::StatusRole);
+        m_model->setData(m_filterModel->mapToSource(m_view->currentIndex()), Transfers::Paused, Transfer::StatusRole);
     }
 }
 
 void MainWindow::removeCurrentTransfer() {
     if (m_view->currentIndex().isValid()) {
-        m_model->setData(m_view->currentIndex(), Transfers::Cancelled, Transfer::StatusRole);
+        m_model->setData(m_filterModel->mapToSource(m_view->currentIndex()), Transfers::Cancelled, Transfer::StatusRole);
     }
 }
 
 void MainWindow::setCurrentTransferConnections() {
     if (m_view->currentIndex().isValid()) {
         if (QAction *action = m_transferConnectionsGroup->checkedAction()) {
-            m_model->setData(m_view->currentIndex(), action->data().toInt(), Transfer::PreferredConnectionsRole);
+            m_model->setData(m_filterModel->mapToSource(m_view->currentIndex()), action->data().toInt(), Transfer::PreferredConnectionsRole);
         }
     }
 }
@@ -682,7 +744,7 @@ void MainWindow::setCurrentTransferConnections() {
 void MainWindow::setCurrentTransferCategory() {
     if (m_view->currentIndex().isValid()) {
         if (QAction *action = qobject_cast<QAction*>(this->sender())) {
-            m_model->setData(m_view->currentIndex(), action->text(), Transfer::CategoryRole);
+            m_model->setData(m_filterModel->mapToSource(m_view->currentIndex()), action->text(), Transfer::CategoryRole);
         }
     }
 }
@@ -690,13 +752,13 @@ void MainWindow::setCurrentTransferCategory() {
 void MainWindow::setCurrentTransferPriority() {
     if (m_view->currentIndex().isValid()) {
         if (m_transferPriorityGroup->checkedAction() == m_transferHighPriorityAction) {
-            m_model->setData(m_view->currentIndex(), Transfers::HighPriority, Transfer::PriorityRole);
+            m_model->setData(m_filterModel->mapToSource(m_view->currentIndex()), Transfers::HighPriority, Transfer::PriorityRole);
         }
         else if (m_transferPriorityGroup->checkedAction() == m_transferLowPriorityAction) {
-            m_model->setData(m_view->currentIndex(), Transfers::LowPriority, Transfer::PriorityRole);
+            m_model->setData(m_filterModel->mapToSource(m_view->currentIndex()), Transfers::LowPriority, Transfer::PriorityRole);
         }
         else {
-            m_model->setData(m_view->currentIndex(), Transfers::NormalPriority, Transfer::PriorityRole);
+            m_model->setData(m_filterModel->mapToSource(m_view->currentIndex()), Transfers::NormalPriority, Transfer::PriorityRole);
         }
     }
 }
@@ -712,7 +774,7 @@ void MainWindow::showCurrentPackageProperties() {
             index = m_view->currentIndex();
         }
 
-        if (Transfer *package = m_model->get(index)) {
+        if (Transfer *package = m_model->get(m_filterModel->mapToSource(index))) {
             PackagePropertiesDialog *dialog = new PackagePropertiesDialog(package, this);
             dialog->open();
         }
@@ -721,21 +783,24 @@ void MainWindow::showCurrentPackageProperties() {
 
 void MainWindow::startCurrentPackage() {
     if (m_view->currentIndex().isValid()) {
-        m_model->setData(m_view->currentIndex().parent().isValid() ? m_view->currentIndex().parent() : m_view->currentIndex(),
+        m_model->setData(m_filterModel->mapToSource(m_view->currentIndex().parent().isValid() ? m_view->currentIndex().parent()
+                                                                                              : m_view->currentIndex()),
                          Transfers::Queued, Transfer::PackageStatusRole);
     }
 }
 
 void MainWindow::pauseCurrentPackage() {
     if (m_view->currentIndex().isValid()) {
-        m_model->setData(m_view->currentIndex().parent().isValid() ? m_view->currentIndex().parent() : m_view->currentIndex(),
+        m_model->setData(m_filterModel->mapToSource(m_view->currentIndex().parent().isValid() ? m_view->currentIndex().parent()
+                                                                                              : m_view->currentIndex()),
                          Transfers::Paused, Transfer::PackageStatusRole);
     }
 }
 
 void MainWindow::removeCurrentPackage() {
     if (m_view->currentIndex().isValid()) {
-        m_model->setData(m_view->currentIndex().parent().isValid() ? m_view->currentIndex().parent() : m_view->currentIndex(),
+        m_model->setData(m_filterModel->mapToSource(m_view->currentIndex().parent().isValid() ? m_view->currentIndex().parent()
+                                                                                              : m_view->currentIndex()),
                          Transfers::Cancelled, Transfer::PackageStatusRole);
     }
 }
@@ -743,7 +808,8 @@ void MainWindow::removeCurrentPackage() {
 void MainWindow::setCurrentPackageCategory() {
     if (m_view->currentIndex().isValid()) {
         if (QAction *action = qobject_cast<QAction*>(this->sender())) {
-            m_model->setData(m_view->currentIndex().parent().isValid() ? m_view->currentIndex().parent() : m_view->currentIndex(),
+            m_model->setData(m_filterModel->mapToSource(m_view->currentIndex().parent().isValid() ? m_view->currentIndex().parent()
+                                                                                                  : m_view->currentIndex()),
                              action->text(), Transfer::CategoryRole);
         }
     }
@@ -752,15 +818,18 @@ void MainWindow::setCurrentPackageCategory() {
 void MainWindow::setCurrentPackagePriority() {
     if (m_view->currentIndex().isValid()) {
         if (m_packagePriorityGroup->checkedAction() == m_packageHighPriorityAction) {
-            m_model->setData(m_view->currentIndex().parent().isValid() ? m_view->currentIndex().parent() : m_view->currentIndex(),
+            m_model->setData(m_filterModel->mapToSource(m_view->currentIndex().parent().isValid() ? m_view->currentIndex().parent()
+                                                                                                  : m_view->currentIndex()),
                              Transfers::HighPriority, Transfer::PriorityRole);
         }
         else if (m_packagePriorityGroup->checkedAction() == m_packageLowPriorityAction) {
-            m_model->setData(m_view->currentIndex().parent().isValid() ? m_view->currentIndex().parent() : m_view->currentIndex(),
+            m_model->setData(m_filterModel->mapToSource(m_view->currentIndex().parent().isValid() ? m_view->currentIndex().parent()
+                                                                                                  : m_view->currentIndex()),
                              Transfers::LowPriority, Transfer::PriorityRole);
         }
         else {
-            m_model->setData(m_view->currentIndex().parent().isValid() ? m_view->currentIndex().parent() : m_view->currentIndex(),
+            m_model->setData(m_filterModel->mapToSource(m_view->currentIndex().parent().isValid() ? m_view->currentIndex().parent()
+                                                                                                  : m_view->currentIndex()),
                              Transfers::NormalPriority, Transfer::PriorityRole);
         }
     }
@@ -825,9 +894,11 @@ void MainWindow::onUrlRetrieverFinished() {
 
 void MainWindow::showTextFileDialog() {
 #if QT_VERSION >= 0x050000
-    QString filePath = QFileDialog::getOpenFileName(this, m_importUrlsAction->text(), QStandardPaths::writableLocation(QStandardPaths::HomeLocation), "*.txt");
+    QString filePath = QFileDialog::getOpenFileName(this, m_importUrlsAction->text(),
+                                                    QStandardPaths::writableLocation(QStandardPaths::HomeLocation), "*.txt");
 #else
-    QString filePath = QFileDialog::getOpenFileName(this, m_importUrlsAction->text(), QDesktopServices::storageLocation(QDesktopServices::HomeLocation), "*.txt");
+    QString filePath = QFileDialog::getOpenFileName(this, m_importUrlsAction->text(),
+                                                    QDesktopServices::storageLocation(QDesktopServices::HomeLocation), "*.txt");
 #endif
     if (!filePath.isEmpty()) {
         this->showAddUrlsDialog(QString(), filePath);
