@@ -20,9 +20,14 @@
 #include "storage.h"
 #include "settings.h"
 #include "urlchecker.h"
+#include "pluginmanager.h"
 #include "utils.h"
 #include <QTimer>
 #include <QCoreApplication>
+#include <QMimeData>
+#ifdef TABLE_TRANSFER_VIEW
+#include <QIcon>
+#endif
 
 TransferModel* TransferModel::self = 0;
 
@@ -30,7 +35,6 @@ TransferModel::TransferModel() :
     QAbstractItemModel(),
     m_rootItem(new Transfer(this)),
     m_queueTimer(new QTimer(this)),
-    m_statusFilter(Transfers::Unknown),
     m_nextAction(Transfers::Continue)
 {
     if (!self) {
@@ -72,9 +76,12 @@ TransferModel::TransferModel() :
     m_queueTimer->setInterval(1000);
 
     this->connect(m_queueTimer, SIGNAL(timeout()), this, SLOT(startNextTransfers()));
-    this->connect(UrlChecker::instance(), SIGNAL(urlReady(QUrl,QString,QString)), this, SLOT(addTransfer(QUrl,QString,QString)));
-    this->connect(Settings::instance(), SIGNAL(maximumConcurrentTransfersChanged(int,int)), this, SLOT(onMaximumConcurrentTransfersChanged(int,int)));
-    this->connect(QCoreApplication::instance(), SIGNAL(aboutToQuit()), this, SLOT(storeAndDeleteTransfers()));
+    this->connect(UrlChecker::instance(), SIGNAL(urlReady(QUrl,QString,QString)),
+                  this, SLOT(addTransfer(QUrl,QString,QString)));
+    this->connect(Settings::instance(), SIGNAL(maximumConcurrentTransfersChanged(int,int)),
+                  this, SLOT(onMaximumConcurrentTransfersChanged(int,int)));
+    this->connect(QCoreApplication::instance(), SIGNAL(aboutToQuit()),
+                  this, SLOT(storeAndDeleteTransfers()));
 }
 
 TransferModel::~TransferModel() {}
@@ -95,14 +102,77 @@ Qt::DropActions TransferModel::supportedDropActions() const {
 
 Qt::ItemFlags TransferModel::flags(const QModelIndex &index) const {
     if (!index.isValid()) {
-        return 0;
+        return Qt::ItemIsDropEnabled;
     }
 
     if (!index.parent().isValid()) {
-        return Qt::ItemIsSelectable | Qt::ItemIsDragEnabled | Qt::ItemIsEnabled;
+        return Qt::ItemIsSelectable | Qt::ItemIsDragEnabled | Qt::ItemIsDropEnabled | Qt::ItemIsEnabled;
     }
 
-    return Qt::ItemIsSelectable | Qt::ItemIsEnabled;
+    return Qt::ItemIsSelectable | Qt::ItemIsDragEnabled | Qt::ItemIsEnabled;
+}
+
+QStringList TransferModel::mimeTypes() const {
+    return QStringList() << "application/x-qdltransfermodeldatalist";
+}
+
+QMimeData* TransferModel::mimeData(const QModelIndexList &indexes) const {
+    if ((indexes.count() <= 0) || (this->mimeTypes().isEmpty())) {
+        return 0;
+    }
+    
+    QMimeData *data = new QMimeData();
+    QByteArray encoded;
+    QDataStream stream(&encoded, QIODevice::WriteOnly);
+    QModelIndexList::ConstIterator it = indexes.begin();
+    
+    for (; it != indexes.end(); it++) {
+        if ((*it).column() == 0) {
+            stream << (*it).row() << ((*it).parent().isValid() ? (*it).parent().row() : -1);
+        }
+    }
+    
+    data->setData(this->mimeTypes().at(0), encoded);
+    return data;
+}
+
+bool TransferModel::dropMimeData(const QMimeData *data, Qt::DropAction action,
+                                 int row, int column, const QModelIndex &parent) {
+    
+    Q_UNUSED(column)                             
+                                 
+    if ((!data) || (action != Qt::MoveAction) || (this->mimeTypes().isEmpty())) {
+        return false;
+    }
+    
+    QString format = this->mimeTypes().at(0);
+    
+    if (!data->hasFormat(format)) {
+        return false;
+    }
+    
+    if ((row < 0) || (row > this->rowCount(parent))) {
+        row = this->rowCount(parent);
+    }
+    
+    QByteArray encoded = data->data(format);
+    QDataStream stream(&encoded, QIODevice::ReadOnly);
+    QList< QPair<int, int> > rows;
+    
+    while (!stream.atEnd()) {
+        int r;
+        int pr;
+        stream >> r >> pr;
+        rows.append(QPair<int, int>(r, pr));
+    }
+    
+    for (int i = 0; i < rows.size(); i++) {
+        const int r = rows.at(i).first;
+        const int pr = rows.at(i).second;
+        this->move(pr == -1 ? QModelIndex() : this->index(pr, 0), r, parent, row);
+    }
+    
+    return true;
 }
 
 int TransferModel::rowCount(const QModelIndex &parent) const {
@@ -121,8 +191,11 @@ int TransferModel::rowCount(const QModelIndex &parent) const {
 
 int TransferModel::columnCount(const QModelIndex &parent) const {
     Q_UNUSED(parent)
-
+#ifdef TABLE_TRANSFER_VIEW
+    return 6;
+#else
     return 1;
+#endif
 }
 
 QModelIndex TransferModel::index(int row, int column, const QModelIndex &parent) const {
@@ -165,16 +238,75 @@ QModelIndex TransferModel::parent(const QModelIndex &child) const {
 
     Transfer *parentTransfer = transfer->parentTransfer();
 
-    if (parentTransfer == m_rootItem) {
+    if ((!parentTransfer) || (parentTransfer == m_rootItem)) {
         return QModelIndex();
     }
 
     return this->createIndex(parentTransfer->rowNumber(), 0, parentTransfer);
 }
 
+#ifdef TABLE_TRANSFER_VIEW
+QVariant TransferModel::headerData(int section, Qt::Orientation orientation, int role) const {
+    if ((orientation != Qt::Horizontal) || (role != Qt::DisplayRole)) {
+        return QVariant();
+    }
+    
+    switch (section) {
+    case 0:
+        return tr("Name");
+    case 1:
+        return tr("Category");
+    case 2:
+        return tr("Connections");
+    case 3:
+        return tr("Priority");
+    case 4:
+        return tr("Progress");
+    case 5:
+        return tr("Status");
+    default:
+        return QVariant();
+    }
+}
+#endif
+
 QVariant TransferModel::data(const QModelIndex &index, int role) const {
     if (index.isValid()) {
         if (Transfer *transfer = this->get(index)) {
+#ifdef TABLE_TRANSFER_VIEW
+            switch (role) {
+            case Qt::DisplayRole:
+                switch (index.column()) {
+                case 0:
+                    return transfer->data(Transfer::NameRole);
+                case 1:
+                    return transfer->data(Transfer::CategoryRole);
+                case 2:
+                    return transfer->data(Transfer::PreferredConnectionsRole);
+                case 3:
+                    return transfer->data(Transfer::PriorityStringRole);
+                case 4:
+                    return transfer->data(Transfer::ProgressRole);
+                case 5:
+                    return transfer->data(Transfer::StatusStringRole);
+                default:
+                    return QVariant();
+                }
+                
+                break;
+            case Qt::DecorationRole:
+                switch (index.column()) {
+                case 0:
+                    return QIcon(transfer->data(Transfer::IconRole).toString());
+                default:
+                    return QVariant();
+                }
+                
+                break;
+            default:
+                break;
+            }
+#endif
             return transfer->data(role);
         }
     }
@@ -183,7 +315,8 @@ QVariant TransferModel::data(const QModelIndex &index, int role) const {
 }
 
 QVariant TransferModel::data(int row, int parentRow, const QByteArray &role) const {
-    return this->data(this->index(row, 0, parentRow >= 0 ? this->index(parentRow, 0) : QModelIndex()), this->roleNames().key(role));
+    return this->data(this->index(row, 0, parentRow >= 0 ? this->index(parentRow, 0)
+                                                         : QModelIndex()), this->roleNames().key(role));
 }
 
 QVariant TransferModel::data(const QString &id, const QByteArray &role) const {
@@ -334,7 +467,8 @@ bool TransferModel::setData(const QModelIndex &index, const QVariant &value, int
 }
 
 bool TransferModel::setData(int row, int parentRow, const QVariant &value, const QByteArray &role) {
-    return this->setData(this->index(row, 0, parentRow >= 0 ? this->index(parentRow, 0) : QModelIndex()), value, this->roleNames().key(role));
+    return this->setData(this->index(row, 0, parentRow >= 0 ? this->index(parentRow, 0)
+                                                            : QModelIndex()), value, this->roleNames().key(role));
 }
 
 bool TransferModel::setData(const QString &id, const QVariant &value, const QByteArray &role) {
@@ -369,7 +503,39 @@ Transfer* TransferModel::get(const QString &id) const {
     return 0;
 }
 
-QModelIndexList TransferModel::match(const QModelIndex &start, int role, const QVariant &value, int hits, Qt::MatchFlags flags) const {
+void TransferModel::move(const QModelIndex &sourceParent, int sourceRow,
+                         const QModelIndex &destinationParent, int destinationRow) {
+             
+    Transfer *parentTransfer = this->get(sourceParent);
+    
+    if (parentTransfer) {
+        this->beginMoveRows(sourceParent, sourceRow, sourceRow, destinationParent, destinationRow);
+        Transfer *transfer = parentTransfer->removeChildTransfer(sourceRow);
+        
+        if (transfer) {
+            parentTransfer = this->get(destinationParent);
+            
+            if (parentTransfer) {
+                parentTransfer->insertChildTransfer(destinationRow, transfer);
+            }
+        }
+        
+        this->endMoveRows();
+    }
+}
+
+void TransferModel::move(int sourceParentRow, int sourceRow, int destinationParentRow, int destinationRow) {
+    const QModelIndex sourceParent = (sourceParentRow == -1 ? QModelIndex()
+                                                            : this->index(sourceParentRow, 0, QModelIndex()));
+    const QModelIndex destinationParent = (destinationParentRow == -1 ? QModelIndex()
+                                                                      : this->index(destinationParentRow, 0, QModelIndex()));
+    
+    this->move(sourceParent, sourceRow, destinationParent, destinationRow);
+}
+
+QModelIndexList TransferModel::match(const QModelIndex &start, int role, const QVariant &value,
+                                     int hits, Qt::MatchFlags flags) const {
+                                     
     Q_UNUSED(flags)
 
     QModelIndexList matches;
@@ -389,35 +555,6 @@ QModelIndexList TransferModel::match(const QModelIndex &start, int role, const Q
     return matches;
 }
 
-QString TransferModel::searchQuery() const {
-    return m_searchQuery;
-}
-
-void TransferModel::setSearchQuery(const QString &query) {
-    if (query != this->searchQuery()) {
-        m_searchQuery = query;
-        this->filter();
-        emit searchQueryChanged(query);
-    }
-}
-
-Transfers::Status TransferModel::statusFilter() const {
-    return m_statusFilter;
-}
-
-void TransferModel::setStatusFilter(Transfers::Status status) {
-    if (status != this->statusFilter()) {
-        m_statusFilter = status;
-        this->filter();
-        emit statusFilterChanged(status);
-    }
-}
-
-void TransferModel::resetFilters() {
-    this->setSearchQuery(QString());
-    this->setStatusFilter(Transfers::Unknown);
-}
-
 Transfers::Action TransferModel::nextAction() const {
     return m_nextAction;
 }
@@ -426,31 +563,11 @@ void TransferModel::setNextAction(Transfers::Action action) {
     if (action != this->nextAction()) {
         m_nextAction = action;
         emit nextActionChanged(action);
-    }
-}
-
-void TransferModel::filter() {
-    int pos = m_filteredTransfers.size() - 1;
-
-    for (int i = m_rootItem->count() - 1; i >= 0; i--) {
-        if ((!m_rootItem->childTransfer(i)->match(Transfer::StatusRole, this->statusFilter())) || (!m_rootItem->childTransfer(i)->match(Transfer::NameRole, this->searchQuery()))) {
-            this->beginRemoveRows(QModelIndex(), i, i);
-            m_filteredTransfers.append(m_rootItem->removeChildTransfer(i));
-            this->endRemoveRows();
-        }
-    }
-
-    for (int i = pos; i >= 0; i--) {
-        if (Transfer *transfer = m_filteredTransfers.at(i)) {
-            if ((transfer->match(Transfer::StatusRole, this->statusFilter())) && (transfer->match(Transfer::NameRole, this->searchQuery()))) {
-                this->beginInsertRows(QModelIndex(), transfer->rowNumber(), transfer->rowNumber());
-                m_filteredTransfers.removeAt(i);
-                m_rootItem->insertChildTransfer(transfer->rowNumber(), transfer);
-                this->endInsertRows();
-            }
-        }
-        else {
-            m_filteredTransfers.removeAt(i);
+        
+        if ((action == Transfers::Continue)
+            && (m_activeTransfers.size() < Settings::instance()->maximumConcurrentTransfers())) {
+            
+            this->getNextTransfers();
         }
     }
 }
@@ -486,7 +603,7 @@ void TransferModel::addTransfer(const QUrl &url, const QString &service, const Q
             packageFound = parentTransfer->transferBelongsToPackage(transfer);
 
             if (packageFound) {
-                int count = parentTransfer->count();
+                const int count = parentTransfer->count();
                 this->beginInsertRows(this->index(i, 0), count, count);
                 parentTransfer->addChildTransfer(transfer);
                 this->endInsertRows();
@@ -512,8 +629,10 @@ void TransferModel::addTransfer(const QUrl &url, const QString &service, const Q
     this->connect(transfer, SIGNAL(statusChanged(Transfers::Status)), this, SLOT(onTransferStatusChanged(Transfers::Status)));
 
     if ((Settings::instance()->startTransfersAutomatically()) && (m_activeTransfers.size() < Settings::instance()->maximumConcurrentTransfers())) {
-        this->addActiveTransfer(transfer);
-        transfer->start();
+        if (this->canAddActiveTransfer(transfer)) {
+            this->addActiveTransfer(transfer);
+            transfer->start();
+        }
     }
 }
 
@@ -524,7 +643,9 @@ void TransferModel::getNextTransfers() {
         foreach (Transfer *parentTransfer, m_rootItem->childTransfers()) {
             if ((parentTransfer->priority() == priority) && (parentTransfer->status() == Transfers::Queued)) {
                 if (m_activeTransfers.size() < Settings::instance()->maximumConcurrentTransfers()) {
-                    this->addActiveTransfer(parentTransfer);
+                    if (this->canAddActiveTransfer(parentTransfer)) {
+                        this->addActiveTransfer(parentTransfer);
+                    }
                 }
                 else {
                     return;
@@ -534,29 +655,9 @@ void TransferModel::getNextTransfers() {
             foreach (Transfer *transfer, parentTransfer->childTransfers()) {
                 if ((transfer->priority() == priority) && (transfer->status() == Transfers::Queued)) {
                     if (m_activeTransfers.size() < Settings::instance()->maximumConcurrentTransfers()) {
-                        this->addActiveTransfer(transfer);
-                    }
-                    else {
-                        return;
-                    }
-                }
-            }
-        }
-
-        foreach (Transfer *parentTransfer, m_filteredTransfers) {
-            if ((parentTransfer->priority() == priority) && (parentTransfer->status() == Transfers::Queued)) {
-                if (m_activeTransfers.size() < Settings::instance()->maximumConcurrentTransfers()) {
-                    this->addActiveTransfer(parentTransfer);
-                }
-                else {
-                    return;
-                }
-            }
-
-            foreach (Transfer *transfer, parentTransfer->childTransfers()) {
-                if ((transfer->priority() == priority) && (transfer->status() == Transfers::Queued)) {
-                    if (m_activeTransfers.size() < Settings::instance()->maximumConcurrentTransfers()) {
-                        this->addActiveTransfer(transfer);
+                        if (this->canAddActiveTransfer(transfer)) {
+                            this->addActiveTransfer(transfer);
+                        }
                     }
                     else {
                         return;
@@ -577,6 +678,26 @@ void TransferModel::startNextTransfers() {
     }
 }
 
+bool TransferModel::canAddActiveTransfer(Transfer *transfer) {
+    if (ServicePlugin *plugin = PluginManager::instance()->getServicePlugin(transfer->serviceName())) {
+        if (plugin->maximumConnections() > 0) {
+            int count = 0;
+            
+            foreach (Transfer *t, m_activeTransfers) {
+                if (t->serviceName() == transfer->serviceName()) {
+                    count++;
+                }
+            }
+            
+            if (count >= plugin->maximumConnections()) {
+                return false;
+            }
+        }
+    }
+    
+    return true;
+}
+
 void TransferModel::addActiveTransfer(Transfer *transfer) {
     m_activeTransfers.append(transfer);
     emit activeTransfersChanged(this->activeTransfers());
@@ -644,6 +765,46 @@ void TransferModel::onTransferSpeedChanged() {
 }
 #else
 void TransferModel::onTransferDataChanged(int role) {
+#ifdef TABLE_TRANSFER_VIEW
+    if (Transfer *transfer = qobject_cast<Transfer*>(this->sender())) {
+        if (Transfer *parentTransfer = transfer->parentTransfer()) {
+            QModelIndex index;
+            int column = 0;
+            
+            switch (role) {
+            case Transfer::ProgressRole:
+            case Transfer::PositionRole:
+            case Transfer::SizeRole:
+                column = 4;
+                break;
+            case Transfer::StatusRole:
+                column = 5;
+                break;
+            case Transfer::CategoryRole:
+                column = 1;
+                break;
+            case Transfer::PreferredConnectionsRole:
+                column = 2;
+                break;
+            case Transfer::PriorityRole:
+                column = 3;
+                break;
+            default:
+                break;
+            }
+
+            if (parentTransfer == m_rootItem) {
+                index = this->index(transfer->rowNumber(), column);
+            }
+            else {
+                index = this->index(transfer->rowNumber(), column, this->index(parentTransfer->rowNumber(), column));
+            }
+
+            emit dataChanged(index, index);
+            emit totalDownloadSpeedChanged(this->totalDownloadSpeed());
+        }
+    }
+#else
     Q_UNUSED(role)
 
     if (Transfer *transfer = qobject_cast<Transfer*>(this->sender())) {
@@ -661,6 +822,7 @@ void TransferModel::onTransferDataChanged(int role) {
             emit totalDownloadSpeedChanged(this->totalDownloadSpeed());
         }
     }
+#endif
 }
 #endif
 
@@ -719,11 +881,11 @@ void TransferModel::onTransferStatusChanged(Transfers::Status status) {
                     this->beginRemoveRows(QModelIndex(), transfer->rowNumber(), transfer->rowNumber());
                 }
                 else {
-                    this->beginRemoveRows(this->index(transfer->parentTransfer()->rowNumber(), 0), transfer->rowNumber(), transfer->rowNumber());
+                    this->beginRemoveRows(this->index(transfer->parentTransfer()->rowNumber(), 0),
+                                          transfer->rowNumber(), transfer->rowNumber());
                 }
 
-                transfer->parentTransfer()->removeChildTransfer(transfer);
-
+                transfer->parentTransfer()->removeChildTransfer(transfer->rowNumber());
                 this->endRemoveRows();
             }
 
@@ -738,7 +900,8 @@ void TransferModel::onTransferStatusChanged(Transfers::Status status) {
                             this->beginInsertRows(QModelIndex(), transfer->rowNumber(), transfer->rowNumber());
                         }
                         else {
-                            this->beginInsertRows(this->index(transfer->parentTransfer()->rowNumber(), 0), transfer->rowNumber(), transfer->rowNumber());
+                            this->beginInsertRows(this->index(transfer->parentTransfer()->rowNumber(), 0),
+                                                  transfer->rowNumber(), transfer->rowNumber());
                         }
 
                         transfer->parentTransfer()->insertChildTransfer(transfer->rowNumber(), firstChild);
@@ -749,7 +912,6 @@ void TransferModel::onTransferStatusChanged(Transfers::Status status) {
             }
 
             transfer->deleteLater();
-
             emit countChanged(this->rowCount());
 
             if (status == Transfers::Completed) {
@@ -777,10 +939,6 @@ void TransferModel::onTransferStatusChanged(Transfers::Status status) {
     }
 
     emit totalDownloadSpeedChanged(this->totalDownloadSpeed());
-
-    if ((!this->searchQuery().isEmpty()) && (this->statusFilter() != Transfers::Unknown)) {
-        this->filter();
-    }
 }
 
 void TransferModel::onMaximumConcurrentTransfersChanged(int oldMaximum, int newMaximum) {
@@ -791,17 +949,21 @@ void TransferModel::onMaximumConcurrentTransfersChanged(int oldMaximum, int newM
     }
     else if (newMaximum < oldMaximum) {
         if (newMaximum < m_activeTransfers.size()) {
-            int priority = Transfers::LowPriority;
-
-            while (priority >= Transfers::HighPriority) {
-                foreach (Transfer *transfer, m_activeTransfers) {
-                    if (transfer->priority() == priority) {
-                        transfer->pause();
-                        return;
+            const int diff = qMin(m_activeTransfers.size(), oldMaximum) - newMaximum;
+            QSet<Transfer*> paused;
+            
+            for (int i = 0; i < diff; i++) {
+                for (int p = Transfers::LowPriority; p >= Transfers::HighPriority; p--) {
+                    for (int i = m_activeTransfers.size() - 1; i >= 0; i--) {
+                        Transfer *transfer = m_activeTransfers.at(i);
+                        
+                        if ((transfer->priority() == p) && (!paused.contains(transfer))) {
+                            transfer->pause();
+                            paused.insert(transfer);
+                            break;
+                        }
                     }
                 }
-
-                priority--;
             }
         }
     }
@@ -817,7 +979,6 @@ void TransferModel::storeTransfers() {
 }
 
 void TransferModel::storeAndDeleteTransfers() {
-    this->resetFilters(); // Ugly fix
     Storage::storeTransfers(m_rootItem->childTransfers(), true);
 }
 
@@ -838,7 +999,8 @@ void TransferModel::restoreStoredTransfers() {
 #else
         this->connect(transfer, SIGNAL(dataChanged(int)), this, SLOT(onTransferDataChanged(int)));
 #endif
-        this->connect(transfer, SIGNAL(statusChanged(Transfers::Status)), this, SLOT(onTransferStatusChanged(Transfers::Status)));
+        this->connect(transfer, SIGNAL(statusChanged(Transfers::Status)),
+                      this, SLOT(onTransferStatusChanged(Transfers::Status)));
 
         foreach (Transfer *childTransfer, transfer->childTransfers()) {
 #ifdef QML_USER_INTERFACE
@@ -846,7 +1008,8 @@ void TransferModel::restoreStoredTransfers() {
 #else
             this->connect(childTransfer, SIGNAL(dataChanged(int)), this, SLOT(onTransferDataChanged(int)));
 #endif
-            this->connect(childTransfer, SIGNAL(statusChanged(Transfers::Status)), this, SLOT(onTransferStatusChanged(Transfers::Status)));
+            this->connect(childTransfer, SIGNAL(statusChanged(Transfers::Status)),
+                          this, SLOT(onTransferStatusChanged(Transfers::Status)));
         }
     }
 
