@@ -31,14 +31,6 @@ class TransferModel : public QAbstractItemModel
     Q_PROPERTY(int count
                READ rowCount
                NOTIFY countChanged)
-    Q_PROPERTY(QString searchQuery
-               READ searchQuery
-               WRITE setSearchQuery
-               NOTIFY searchQueryChanged)
-    Q_PROPERTY(Transfers::Status statusFilter
-               READ statusFilter
-               WRITE setStatusFilter
-               NOTIFY statusFilterChanged)
     Q_PROPERTY(Transfers::Action nextAction
                READ nextAction
                WRITE setNextAction
@@ -54,14 +46,25 @@ public:
 #if (QT_VERSION >= 0x050000) || (QT_VERSION < 0x040600)
     QHash<int, QByteArray> roleNames() const;
 #endif
+
     Qt::DropActions supportedDropActions() const;
+    
     Qt::ItemFlags flags(const QModelIndex &index) const;
+    
+    QStringList mimeTypes() const;
+    QMimeData* mimeData(const QModelIndexList &indexes) const;
+    bool dropMimeData(const QMimeData *data, Qt::DropAction action,
+                      int row, int column, const QModelIndex &parent);
 
     int rowCount(const QModelIndex &parent = QModelIndex()) const;
     int columnCount(const QModelIndex &parent = QModelIndex()) const;
 
     QModelIndex index(int row, int column, const QModelIndex &parent = QModelIndex()) const;
     QModelIndex parent(const QModelIndex &child) const;
+    
+#ifdef TABLE_TRANSFER_VIEW
+    QVariant headerData(int section, Qt::Orientation orientation = Qt::Horizontal, int role = Qt::DisplayRole) const;
+#endif
 
     QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const;
     Q_INVOKABLE QVariant data(int row, int parentRow, const QByteArray &role) const;
@@ -70,7 +73,8 @@ public:
     QMap<int, QVariant> itemData(const QModelIndex &index) const;
     Q_INVOKABLE QVariantMap itemData(int row, int parentRow) const;
     Q_INVOKABLE QVariantMap itemData(const QString &id) const;
-    Q_INVOKABLE QVariantList allItemData(Transfers::Status filter = Transfers::Unknown, const QString &query = QString(), int start = 0, int count = -1) const;
+    Q_INVOKABLE QVariantList allItemData(Transfers::Status filter = Transfers::Unknown,
+                                         const QString &query = QString(), int start = 0, int count = -1) const;
 
     bool setData(const QModelIndex &index, const QVariant &value, int role);
     Q_INVOKABLE bool setData(int row, int parentRow, const QVariant &value, const QByteArray &role);
@@ -79,11 +83,12 @@ public:
     Transfer* get(const QModelIndex &index) const;
     Q_INVOKABLE Transfer* get(int row, int parentRow) const;
     Q_INVOKABLE Transfer* get(const QString &id) const;
+    
+    void move(const QModelIndex &sourceParent, int sourceRow, const QModelIndex &destinationParent, int destinationRow);
+    Q_INVOKABLE void move(int sourceParentRow, int sourceRow, int destinationParentRow, int destinationRow);
 
-    QModelIndexList match(const QModelIndex &start, int role, const QVariant &value, int hits = -1, Qt::MatchFlags flags = Qt::MatchExactly) const;
-
-    QString searchQuery() const;
-    Transfers::Status statusFilter() const;
+    QModelIndexList match(const QModelIndex &start, int role, const QVariant &value,
+                          int hits = -1, Qt::MatchFlags flags = Qt::MatchExactly) const;
 
     Transfers::Action nextAction() const;
 
@@ -94,10 +99,6 @@ public:
     static TransferModel* instance();
 
 public slots:
-    void setSearchQuery(const QString &query);
-    void setStatusFilter(Transfers::Status status);
-    void resetFilters();
-
     void setNextAction(Transfers::Action action);
 
     void addTransfer(const QUrl &url, const QString &service, const QString &fileName);
@@ -115,10 +116,9 @@ private:
     TransferModel();
     ~TransferModel();
 
-    void filter();
-
     void getNextTransfers();
 
+    bool canAddActiveTransfer(Transfer *transfer);
     void addActiveTransfer(Transfer *transfer);
     void removeActiveTransfer(Transfer *transfer);
 
@@ -135,8 +135,6 @@ private slots:
 
 signals:
     void countChanged(int count);
-    void searchQueryChanged(const QString &query);
-    void statusFilterChanged(Transfers::Status status);
     void nextActionChanged(Transfers::Action action);
     void totalDownloadSpeedChanged(int speed);
     void activeTransfersChanged(int active);
@@ -144,12 +142,9 @@ signals:
 private:
     static TransferModel *self;
 
-    Transfer* m_rootItem;
+    Transfer *m_rootItem;
     QTimer *m_queueTimer;
     QList<Transfer*> m_activeTransfers;
-    QList<Transfer*> m_filteredTransfers;
-    QString m_searchQuery;
-    Transfers::Status m_statusFilter;
     Transfers::Action m_nextAction;
     QHash<int, QByteArray> m_roleNames;
 };
