@@ -1,3 +1,20 @@
+/*
+ * Copyright (C) 2014 Stuart Howarth <showarth@marxoft.co.uk>
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms and conditions of the GNU Lesser General Public License,
+ * version 3, as published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope it will be useful, but WITHOUT ANY
+ * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+ * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
+ * more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
+ */
+
 #ifndef KEEPTOSHARE_H
 #define KEEPTOSHARE_H
 
@@ -24,6 +41,7 @@ public:
     inline bool loginSupported() const { return true; }
     void login(const QString &username, const QString &password);
     inline bool recaptchaRequired() const { return true; }
+    inline QString recaptchaServiceName() const { return QString("Keep2Share"); }
     inline QString recaptchaKey() const { return m_captchaKey; }
     inline int maximumConnections() const { return m_connections; }
     bool cancelCurrentOperation();
@@ -32,13 +50,15 @@ public slots:
     void submitCaptchaResponse(const QString &challenge, const QString &response);
 
 private:
-    void loginK2C(const QString &username, const QString &password);
+    void loginK2S(const QString &username, const QString &password);
+    void loginKeep2s(const QString &username, const QString &password);
     void getCaptchaKey();
     void startWait(int msecs);
 
 private slots:
     void checkKeep2ShareLogin();
-    void checkK2CLogin();
+    void checkK2SLogin();
+    void checkKeep2sLogin();
     void checkUrlIsValid();
     void onWebPageDownloaded();
     void checkCaptchaKey();
