@@ -21,7 +21,10 @@
 #include <QNetworkRequest>
 #include <QRegExp>
 #include <QTimer>
-#include <QDateTime>
+#ifdef DOWNLOAD_TEST
+#include <QCoreApplication>
+#include <QDebug>
+#endif
 
 OneFichier::OneFichier(QObject *parent) :
     ServicePlugin(parent),
@@ -30,10 +33,16 @@ OneFichier::OneFichier(QObject *parent) :
     m_connections(1)
 {
     this->connect(m_waitTimer, SIGNAL(timeout()), this, SLOT(updateWaitTime()));
+#ifdef DOWNLOAD_TEST
+    m_downloadReply = 0;
+    this->setNetworkAccessManager(new QNetworkAccessManager(this));
+    this->connect(this, SIGNAL(downloadRequestReady(QNetworkRequest)), this, SLOT(onDownloadRequestReady(QNetworkRequest)));
+    this->connect(this, SIGNAL(error(ServicePlugin::ErrorType)), this, SLOT(onDownloadError(ServicePlugin::ErrorType)));
+#endif
 }
 
 QRegExp OneFichier::urlPattern() const {
-    return QRegExp("http(s|)://\\w+.1fichier.com", Qt::CaseInsensitive);
+    return QRegExp("http(s|)://(\\w+.1fichier.com|1fichier.com/\\?\\w+)", Qt::CaseInsensitive);
 }
 
 bool OneFichier::urlSupported(const QUrl &url) const {
@@ -42,7 +51,7 @@ bool OneFichier::urlSupported(const QUrl &url) const {
 
 void OneFichier::login(const QString &username, const QString &password) {
     QString data = QString("mail=%1&pass=%2").arg(username).arg(password);
-    QUrl url("http://1fichier.com/login.pl");
+    QUrl url("https://1fichier.com/login.pl");
     QNetworkRequest request(url);
     request.setHeader(QNetworkRequest::ContentTypeHeader, "application/x-www-form-urlencoded");
     QNetworkReply *reply = this->networkAccessManager()->post(request, data.toUtf8());
@@ -76,18 +85,8 @@ void OneFichier::checkLogin() {
     reply->deleteLater();
 }
 
-void OneFichier::checkUrl(const QUrl &webUrl) {
-    QString urlString = webUrl.toString();
-    QNetworkRequest request;
-
-    if (!urlString.endsWith("/en")) {
-        urlString = urlString.section("1fichier.com", 0, 0).append("1fichier.com/en");
-        request.setUrl(QUrl(urlString));
-    }
-    else {
-        request.setUrl(webUrl);
-    }
-
+void OneFichier::checkUrl(const QUrl &url) {
+    QNetworkRequest request(url);
     request.setRawHeader("Accept-Language", "en-GB,en-US;q=0.8,en;q=0.6");
     QNetworkReply *reply = this->networkAccessManager()->get(request);
     this->connect(reply, SIGNAL(finished()), this, SLOT(checkUrlIsValid()));
@@ -103,14 +102,16 @@ void OneFichier::checkUrlIsValid() {
     }
 
     QString redirect = reply->attribute(QNetworkRequest::RedirectionTargetAttribute).toString();
-    QRegExp re("http://\\w-\\d.1fichier.com/get/[^'\"]+");
+    QRegExp re("http(s|)://\\w-\\d.1fichier.com/[^'\"]+");
 
     if ((!redirect.isEmpty()) && (re.indexIn(redirect) == -1)) {
         this->checkUrl(QUrl(redirect));
     }
     else {
         QString response(reply->readAll());
-        QString fileName = response.section("Filename :</th><td>", 1, 1).section('<', 0, 0);
+        QString fileName = response.section("FileName :</td>", 1, 1)
+                                   .section("class=\"normal\">", 1, 1)
+                                   .section('<', 0, 0);
 
         if (fileName.isEmpty()) {
             emit urlChecked(false);
@@ -123,9 +124,9 @@ void OneFichier::checkUrlIsValid() {
     reply->deleteLater();
 }
 
-void OneFichier::getDownloadRequest(const QUrl &webUrl) {
+void OneFichier::getDownloadRequest(const QUrl &url) {
     emit statusChanged(Connecting);
-    QNetworkRequest request(webUrl);
+    QNetworkRequest request(url);
     request.setRawHeader("Accept-Language", "en-GB,en-US;q=0.8,en;q=0.6");
     QNetworkReply *reply = this->networkAccessManager()->get(request);
     this->connect(reply, SIGNAL(finished()), this, SLOT(onWebPageDownloaded()));
@@ -140,7 +141,7 @@ void OneFichier::onWebPageDownloaded() {
         return;
     }
 
-    QRegExp re("http://\\w-\\d.1fichier.com/get/[^'\"]+");
+    QRegExp re("http(s|)://\\w-\\d.1fichier.com/[^'\"]+");
     QString redirect = reply->attribute(QNetworkRequest::RedirectionTargetAttribute).toString();
 
     if (re.indexIn(redirect) == 0) {
@@ -159,12 +160,23 @@ void OneFichier::onWebPageDownloaded() {
             request.setUrl(QUrl(re.cap()));
             emit downloadRequestReady(request);
         }
-        else if (response.contains("you must wait between each download")) {
-            this->startWait(300000);
-            this->connect(this, SIGNAL(waitFinished()), this, SLOT(onWaitFinished()));
-        }
         else {
-            this->getDownloadLink(reply->request().url());
+            QRegExp re("must wait \\d+ minutes");
+        
+            if (re.indexIn(response) >= 0) {
+                int mins = re.cap().section("must wait ", 1, 1).section(' ', 0, 0).toInt();
+                
+                if (mins > 0) {
+                    this->startWait(mins * 61000);
+                    this->connect(this, SIGNAL(waitFinished()), this, SLOT(onWaitFinished()));
+                }
+                else {
+                    emit error(UnknownError);
+                }
+            }
+            else {
+                this->getDownloadLink(reply->request().url());
+            }
         }
     }
 
@@ -174,8 +186,11 @@ void OneFichier::onWebPageDownloaded() {
 void OneFichier::getDownloadLink(const QUrl &url) {
     QNetworkRequest request(url);
     request.setHeader(QNetworkRequest::ContentTypeHeader, "application/x-www-form-urlencoded");
-    request.setRawHeader("Accept-Language", "en-GB,en-US;q=0.8,en;q=0.6");
-    QNetworkReply *reply = this->networkAccessManager()->post(request, QByteArray("sub=1&submit=Download"));
+    //request.setRawHeader("Referer", url.toString().toUtf8());
+    //request.setRawHeader("Accept-Language", "en-GB,en-US;q=0.8,en;q=0.6");
+    request.setRawHeader("User-Agent", "Wget/1.13.4 (linux-gnu)");
+    request.setRawHeader("Connection", "Keep-Alive");
+    QNetworkReply *reply = this->networkAccessManager()->post(request, QByteArray());
     this->connect(reply, SIGNAL(finished()), this, SLOT(checkDownloadLink()));
     this->connect(this, SIGNAL(currentOperationCancelled()), reply, SLOT(deleteLater()));
 }
@@ -195,10 +210,18 @@ void OneFichier::checkDownloadLink() {
     }
     else {
         QString response(reply->readAll());
-
-        if (response.contains("you must wait up to 15 minutes between each download")) {
-            this->startWait(300000);
-            this->connect(this, SIGNAL(waitFinished()), this, SLOT(onWaitFinished()));
+        QRegExp re("must wait \\d+ minutes");
+        
+        if (re.indexIn(response) >= 0) {
+            int mins = re.cap().section("must wait ", 1, 1).section(' ', 0, 0).toInt();
+            
+            if (mins > 0) {
+                this->startWait(mins * 61000);
+                this->connect(this, SIGNAL(waitFinished()), this, SLOT(onWaitFinished()));
+            }
+            else {
+                emit error(UnknownError);
+            }
         }
         else {
             emit error(UnknownError);
@@ -209,6 +232,10 @@ void OneFichier::checkDownloadLink() {
 }
 
 void OneFichier::startWait(int msecs) {
+#ifdef DOWNLOAD_TEST
+    qDebug() << "Need to wait" << msecs << "before download";
+    QCoreApplication::instance()->quit();
+#else
     if (msecs > 60000) {
         emit statusChanged(LongWait);
     }
@@ -219,6 +246,7 @@ void OneFichier::startWait(int msecs) {
     emit waiting(msecs);
     m_waitTime = msecs;
     m_waitTimer->start(1000);
+#endif
 }
 
 void OneFichier::updateWaitTime() {
@@ -244,4 +272,63 @@ bool OneFichier::cancelCurrentOperation() {
     return true;
 }
 
+#ifdef DOWNLOAD_TEST
+void OneFichier::downloadTest(const QUrl &url, const QString &fileName) {
+    m_downloadFile.setFileName(fileName);
+    this->getDownloadLink(url);
+}
+    
+void OneFichier::onDownloadRequestReady(const QNetworkRequest &request) {
+    if (m_downloadReply) {
+        delete m_downloadReply;
+    }
+    
+    if (!m_downloadFile.open(QIODevice::WriteOnly)) {
+        qDebug() << "Download error:" << m_downloadFile.errorString();
+        QCoreApplication::instance()->quit();
+    }
+    
+    m_downloadReply = this->networkAccessManager()->get(request);
+    this->connect(m_downloadReply, SIGNAL(downloadProgress(qint64,qint64)), this, SLOT(onDownloadProgressChanged(qint64,qint64)));
+    this->connect(m_downloadReply, SIGNAL(readyRead()), this, SLOT(onDownloadReadyRead()));
+    this->connect(m_downloadReply, SIGNAL(finished()), this, SLOT(onDownloadFinished()));
+    this->connect(m_downloadReply, SIGNAL(sslErrors(QList<QSslError>)), this, SLOT(onDownloadSSLErrors(QList<QSslError>)));
+}
+
+void OneFichier::onDownloadProgressChanged(qint64 received, qint64 total) {
+    qDebug() << "Download progress:" << received << "/" << total;
+}
+
+void OneFichier::onDownloadReadyRead() {
+    m_downloadFile.write(m_downloadReply->readAll());
+}
+
+void OneFichier::onDownloadFinished() {
+    m_downloadFile.close();
+    
+    switch (m_downloadReply->error()) {
+    case QNetworkReply::NoError:
+        qDebug() << "Download completed successfully at" << m_downloadFile.fileName();
+        break;
+    default:
+        qDebug() << "Download error:" << m_downloadReply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt() << m_downloadReply->errorString();
+        break;
+    }
+    
+    m_downloadReply->deleteLater();
+    m_downloadReply = 0;
+    //QCoreApplication::instance()->quit();
+}
+
+void OneFichier::onDownloadError(ServicePlugin::ErrorType e) {
+    qDebug() << "Download error:" << e;
+    QCoreApplication::instance()->quit();
+}
+
+void OneFichier::onDownloadSSLErrors(const QList<QSslError> &errors) {
+    qDebug() << "SSL errors:" << errors;
+    m_downloadReply->ignoreSslErrors(errors);
+}
+#endif
+
 Q_EXPORT_PLUGIN2(onefichier, OneFichier)
